# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .task_status import TaskStatus
from .package_credentials import PackageCredentials

__all__ = ["Task"]


class Task(BaseModel):
    device_id: str = FieldInfo(alias="deviceId")

    llm_model: str = FieldInfo(alias="llmModel")
    """The LLM model identifier to use for the task (e.g. 'gemini/gemini-2.5-flash')"""

    task: str

    user_id: str = FieldInfo(alias="userId")

    id: Optional[str] = None

    agent_id: Optional[int] = FieldInfo(alias="agentId", default=None)

    apps: Optional[List[str]] = None

    created_at: Optional[datetime] = FieldInfo(alias="createdAt", default=None)

    credentials: Optional[List[PackageCredentials]] = None

    display_id: Optional[int] = FieldInfo(alias="displayId", default=None)

    execution_timeout: Optional[int] = FieldInfo(alias="executionTimeout", default=None)

    files: Optional[List[str]] = None

    finished_at: Optional[datetime] = FieldInfo(alias="finishedAt", default=None)

    max_steps: Optional[int] = FieldInfo(alias="maxSteps", default=None)

    message: Optional[str] = None

    output: Optional[Dict[str, object]] = None

    output_schema: Optional[Dict[str, object]] = FieldInfo(alias="outputSchema", default=None)

    reasoning: Optional[bool] = None

    status: Optional[TaskStatus] = None

    stealth: Optional[bool] = None

    steps: Optional[int] = None

    succeeded: Optional[bool] = None

    temperature: Optional[float] = None

    tmp_device: Optional[bool] = FieldInfo(alias="tmpDevice", default=None)

    trajectory: Optional[List[Dict[str, object]]] = None

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)

    vision: Optional[bool] = None

    vpn_country: Optional[Literal["US", "BR", "FR", "DE", "IN", "JP", "KR", "ZA"]] = FieldInfo(
        alias="vpnCountry", default=None
    )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import proxy_create_params, proxy_update_params
from .._types import Body, Query, Headers, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.proxy_list_response import ProxyListResponse
from ..types.proxy_create_response import ProxyCreateResponse
from ..types.proxy_delete_response import ProxyDeleteResponse
from ..types.proxy_update_response import ProxyUpdateResponse
from ..types.proxy_retrieve_response import ProxyRetrieveResponse

__all__ = ["ProxiesResource", "AsyncProxiesResource"]


class ProxiesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ProxiesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#accessing-raw-response-data-eg-headers
        """
        return ProxiesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProxiesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#with_streaming_response
        """
        return ProxiesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        host: str,
        name: str,
        password: str,
        port: int,
        user: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProxyCreateResponse:
        """
        Create a new proxy config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/proxies",
            body=maybe_transform(
                {
                    "host": host,
                    "name": name,
                    "password": password,
                    "port": port,
                    "user": user,
                },
                proxy_create_params.ProxyCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxyCreateResponse,
        )

    def retrieve(
        self,
        proxy_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProxyRetrieveResponse:
        """
        Get a specific proxy config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not proxy_id:
            raise ValueError(f"Expected a non-empty value for `proxy_id` but received {proxy_id!r}")
        return self._get(
            f"/proxies/{proxy_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxyRetrieveResponse,
        )

    def update(
        self,
        proxy_id: str,
        *,
        host: str,
        name: str,
        password: str,
        port: int,
        user: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProxyUpdateResponse:
        """
        Update a proxy config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not proxy_id:
            raise ValueError(f"Expected a non-empty value for `proxy_id` but received {proxy_id!r}")
        return self._put(
            f"/proxies/{proxy_id}",
            body=maybe_transform(
                {
                    "host": host,
                    "name": name,
                    "password": password,
                    "port": port,
                    "user": user,
                },
                proxy_update_params.ProxyUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxyUpdateResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProxyListResponse:
        """List all proxy configs for the authenticated user"""
        return self._get(
            "/proxies",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxyListResponse,
        )

    def delete(
        self,
        proxy_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProxyDeleteResponse:
        """
        Delete a proxy config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not proxy_id:
            raise ValueError(f"Expected a non-empty value for `proxy_id` but received {proxy_id!r}")
        return self._delete(
            f"/proxies/{proxy_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxyDeleteResponse,
        )


class AsyncProxiesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncProxiesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncProxiesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProxiesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#with_streaming_response
        """
        return AsyncProxiesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        host: str,
        name: str,
        password: str,
        port: int,
        user: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProxyCreateResponse:
        """
        Create a new proxy config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/proxies",
            body=await async_maybe_transform(
                {
                    "host": host,
                    "name": name,
                    "password": password,
                    "port": port,
                    "user": user,
                },
                proxy_create_params.ProxyCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxyCreateResponse,
        )

    async def retrieve(
        self,
        proxy_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProxyRetrieveResponse:
        """
        Get a specific proxy config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not proxy_id:
            raise ValueError(f"Expected a non-empty value for `proxy_id` but received {proxy_id!r}")
        return await self._get(
            f"/proxies/{proxy_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxyRetrieveResponse,
        )

    async def update(
        self,
        proxy_id: str,
        *,
        host: str,
        name: str,
        password: str,
        port: int,
        user: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProxyUpdateResponse:
        """
        Update a proxy config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not proxy_id:
            raise ValueError(f"Expected a non-empty value for `proxy_id` but received {proxy_id!r}")
        return await self._put(
            f"/proxies/{proxy_id}",
            body=await async_maybe_transform(
                {
                    "host": host,
                    "name": name,
                    "password": password,
                    "port": port,
                    "user": user,
                },
                proxy_update_params.ProxyUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxyUpdateResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProxyListResponse:
        """List all proxy configs for the authenticated user"""
        return await self._get(
            "/proxies",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxyListResponse,
        )

    async def delete(
        self,
        proxy_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProxyDeleteResponse:
        """
        Delete a proxy config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not proxy_id:
            raise ValueError(f"Expected a non-empty value for `proxy_id` but received {proxy_id!r}")
        return await self._delete(
            f"/proxies/{proxy_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxyDeleteResponse,
        )


class ProxiesResourceWithRawResponse:
    def __init__(self, proxies: ProxiesResource) -> None:
        self._proxies = proxies

        self.create = to_raw_response_wrapper(
            proxies.create,
        )
        self.retrieve = to_raw_response_wrapper(
            proxies.retrieve,
        )
        self.update = to_raw_response_wrapper(
            proxies.update,
        )
        self.list = to_raw_response_wrapper(
            proxies.list,
        )
        self.delete = to_raw_response_wrapper(
            proxies.delete,
        )


class AsyncProxiesResourceWithRawResponse:
    def __init__(self, proxies: AsyncProxiesResource) -> None:
        self._proxies = proxies

        self.create = async_to_raw_response_wrapper(
            proxies.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            proxies.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            proxies.update,
        )
        self.list = async_to_raw_response_wrapper(
            proxies.list,
        )
        self.delete = async_to_raw_response_wrapper(
            proxies.delete,
        )


class ProxiesResourceWithStreamingResponse:
    def __init__(self, proxies: ProxiesResource) -> None:
        self._proxies = proxies

        self.create = to_streamed_response_wrapper(
            proxies.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            proxies.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            proxies.update,
        )
        self.list = to_streamed_response_wrapper(
            proxies.list,
        )
        self.delete = to_streamed_response_wrapper(
            proxies.delete,
        )


class AsyncProxiesResourceWithStreamingResponse:
    def __init__(self, proxies: AsyncProxiesResource) -> None:
        self._proxies = proxies

        self.create = async_to_streamed_response_wrapper(
            proxies.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            proxies.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            proxies.update,
        )
        self.list = async_to_streamed_response_wrapper(
            proxies.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            proxies.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from mobilerun import Mobilerun, AsyncMobilerun
from tests.utils import assert_matches_type
from mobilerun.types import (
    ProxyListResponse,
    ProxyCreateResponse,
    ProxyDeleteResponse,
    ProxyUpdateResponse,
    ProxyRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestProxies:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Mobilerun) -> None:
        proxy = client.proxies.create(
            host="x",
            name="xxx",
            password="x",
            port=1,
            user="x",
        )
        assert_matches_type(ProxyCreateResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Mobilerun) -> None:
        response = client.proxies.with_raw_response.create(
            host="x",
            name="xxx",
            password="x",
            port=1,
            user="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        proxy = response.parse()
        assert_matches_type(ProxyCreateResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Mobilerun) -> None:
        with client.proxies.with_streaming_response.create(
            host="x",
            name="xxx",
            password="x",
            port=1,
            user="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            proxy = response.parse()
            assert_matches_type(ProxyCreateResponse, proxy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Mobilerun) -> None:
        proxy = client.proxies.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(ProxyRetrieveResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Mobilerun) -> None:
        response = client.proxies.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        proxy = response.parse()
        assert_matches_type(ProxyRetrieveResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Mobilerun) -> None:
        with client.proxies.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            proxy = response.parse()
            assert_matches_type(ProxyRetrieveResponse, proxy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `proxy_id` but received ''"):
            client.proxies.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Mobilerun) -> None:
        proxy = client.proxies.update(
            proxy_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            host="x",
            name="xxx",
            password="x",
            port=1,
            user="x",
        )
        assert_matches_type(ProxyUpdateResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Mobilerun) -> None:
        response = client.proxies.with_raw_response.update(
            proxy_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            host="x",
            name="xxx",
            password="x",
            port=1,
            user="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        proxy = response.parse()
        assert_matches_type(ProxyUpdateResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Mobilerun) -> None:
        with client.proxies.with_streaming_response.update(
            proxy_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            host="x",
            name="xxx",
            password="x",
            port=1,
            user="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            proxy = response.parse()
            assert_matches_type(ProxyUpdateResponse, proxy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `proxy_id` but received ''"):
            client.proxies.with_raw_response.update(
                proxy_id="",
                host="x",
                name="xxx",
                password="x",
                port=1,
                user="x",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Mobilerun) -> None:
        proxy = client.proxies.list()
        assert_matches_type(ProxyListResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Mobilerun) -> None:
        response = client.proxies.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        proxy = response.parse()
        assert_matches_type(ProxyListResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Mobilerun) -> None:
        with client.proxies.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            proxy = response.parse()
            assert_matches_type(ProxyListResponse, proxy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Mobilerun) -> None:
        proxy = client.proxies.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(ProxyDeleteResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Mobilerun) -> None:
        response = client.proxies.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        proxy = response.parse()
        assert_matches_type(ProxyDeleteResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Mobilerun) -> None:
        with client.proxies.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            proxy = response.parse()
            assert_matches_type(ProxyDeleteResponse, proxy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `proxy_id` but received ''"):
            client.proxies.with_raw_response.delete(
                "",
            )


class TestAsyncProxies:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncMobilerun) -> None:
        proxy = await async_client.proxies.create(
            host="x",
            name="xxx",
            password="x",
            port=1,
            user="x",
        )
        assert_matches_type(ProxyCreateResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.proxies.with_raw_response.create(
            host="x",
            name="xxx",
            password="x",
            port=1,
            user="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        proxy = await response.parse()
        assert_matches_type(ProxyCreateResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncMobilerun) -> None:
        async with async_client.proxies.with_streaming_response.create(
            host="x",
            name="xxx",
            password="x",
            port=1,
            user="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            proxy = await response.parse()
            assert_matches_type(ProxyCreateResponse, proxy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncMobilerun) -> None:
        proxy = await async_client.proxies.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(ProxyRetrieveResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.proxies.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        proxy = await response.parse()
        assert_matches_type(ProxyRetrieveResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncMobilerun) -> None:
        async with async_client.proxies.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            proxy = await response.parse()
            assert_matches_type(ProxyRetrieveResponse, proxy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `proxy_id` but received ''"):
            await async_client.proxies.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncMobilerun) -> None:
        proxy = await async_client.proxies.update(
            proxy_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            host="x",
            name="xxx",
            password="x",
            port=1,
            user="x",
        )
        assert_matches_type(ProxyUpdateResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.proxies.with_raw_response.update(
            proxy_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            host="x",
            name="xxx",
            password="x",
            port=1,
            user="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        proxy = await response.parse()
        assert_matches_type(ProxyUpdateResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncMobilerun) -> None:
        async with async_client.proxies.with_streaming_response.update(
            proxy_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            host="x",
            name="xxx",
            password="x",
            port=1,
            user="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            proxy = await response.parse()
            assert_matches_type(ProxyUpdateResponse, proxy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `proxy_id` but received ''"):
            await async_client.proxies.with_raw_response.update(
                proxy_id="",
                host="x",
                name="xxx",
                password="x",
                port=1,
                user="x",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncMobilerun) -> None:
        proxy = await async_client.proxies.list()
        assert_matches_type(ProxyListResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.proxies.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        proxy = await response.parse()
        assert_matches_type(ProxyListResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncMobilerun) -> None:
        async with async_client.proxies.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            proxy = await response.parse()
            assert_matches_type(ProxyListResponse, proxy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncMobilerun) -> None:
        proxy = await async_client.proxies.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(ProxyDeleteResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.proxies.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        proxy = await response.parse()
        assert_matches_type(ProxyDeleteResponse, proxy, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncMobilerun) -> None:
        async with async_client.proxies.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            proxy = await response.parse()
            assert_matches_type(ProxyDeleteResponse, proxy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `proxy_id` but received ''"):
            await async_client.proxies.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CarrierCreateParams"]


class CarrierCreateParams(TypedDict, total=False):
    country: Required[str]
    """Country name"""

    mcc: Required[str]
    """Mobile Country Code"""

    mnc: Required[str]
    """Mobile Network Code"""

    operator: Required[str]
    """Operator name"""

    company: str
    """Company name"""

    country_code: str
    """Country dialing code (e.g., +1)"""

    country_iso: str
    """ISO country code"""

    detail_url: str
    """URL to carrier details page"""

    gsm_bands: str
    """Supported GSM bands"""

    lte_bands: str
    """Supported LTE bands"""

    mobile_prefix: str
    """Mobile number prefix"""

    nsn_size: str
    """National Significant Number size"""

    number_format: str
    """Phone number format"""

    protocols: str
    """Supported protocols (comma-separated)"""

    umts_bands: str
    """Supported UMTS bands"""

    website: str
    """Company website"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .packages import (
    PackagesResource,
    AsyncPackagesResource,
    PackagesResourceWithRawResponse,
    AsyncPackagesResourceWithRawResponse,
    PackagesResourceWithStreamingResponse,
    AsyncPackagesResourceWithStreamingResponse,
)
from .credentials import (
    CredentialsResource,
    AsyncCredentialsResource,
    CredentialsResourceWithRawResponse,
    AsyncCredentialsResourceWithRawResponse,
    CredentialsResourceWithStreamingResponse,
    AsyncCredentialsResourceWithStreamingResponse,
)

__all__ = [
    "CredentialsResource",
    "AsyncCredentialsResource",
    "CredentialsResourceWithRawResponse",
    "AsyncCredentialsResourceWithRawResponse",
    "CredentialsResourceWithStreamingResponse",
    "AsyncCredentialsResourceWithStreamingResponse",
    "PackagesResource",
    "AsyncPackagesResource",
    "PackagesResourceWithRawResponse",
    "AsyncPackagesResourceWithRawResponse",
    "PackagesResourceWithStreamingResponse",
    "AsyncPackagesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["PackageCredentialsParam"]


class PackageCredentialsParam(TypedDict, total=False):
    credential_names: Required[Annotated[SequenceNotStr[str], PropertyInfo(alias="credentialNames")]]

    package_name: Required[Annotated[str, PropertyInfo(alias="packageName")]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel
from .proxy_config import ProxyConfig

__all__ = ["ProxyRetrieveResponse"]


class ProxyRetrieveResponse(BaseModel):
    data: ProxyConfig



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .config import Config
from ..._models import BaseModel
from .device_carrier import DeviceCarrier
from .device_identifiers import DeviceIdentifiers

__all__ = ["DeviceSpec"]


class DeviceSpec(BaseModel):
    schema_: Optional[str] = FieldInfo(alias="$schema", default=None)
    """A URL to the JSON Schema for this object."""

    apps: Optional[List[str]] = None

    carrier: Optional[DeviceCarrier] = None

    files: Optional[List[str]] = None

    identifiers: Optional[DeviceIdentifiers] = None

    name: Optional[str] = None

    proxy: Optional[Config] = None

    smart_ip: Optional[bool] = FieldInfo(alias="smartIp", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .shared.pagination import Pagination
from .credentials.packages.credential import Credential

__all__ = ["CredentialListResponse"]


class CredentialListResponse(BaseModel):
    items: List[Credential]

    pagination: Pagination



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from mobilerun import Mobilerun, AsyncMobilerun

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestKeyboard:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_clear(self, client: Mobilerun) -> None:
        keyboard = client.devices.keyboard.clear(
            device_id="deviceId",
        )
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_clear_with_all_params(self, client: Mobilerun) -> None:
        keyboard = client.devices.keyboard.clear(
            device_id="deviceId",
            x_device_display_id=0,
        )
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_clear(self, client: Mobilerun) -> None:
        response = client.devices.keyboard.with_raw_response.clear(
            device_id="deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        keyboard = response.parse()
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_clear(self, client: Mobilerun) -> None:
        with client.devices.keyboard.with_streaming_response.clear(
            device_id="deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            keyboard = response.parse()
            assert keyboard is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_clear(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.keyboard.with_raw_response.clear(
                device_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_key(self, client: Mobilerun) -> None:
        keyboard = client.devices.keyboard.key(
            device_id="deviceId",
            key=0,
        )
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_key_with_all_params(self, client: Mobilerun) -> None:
        keyboard = client.devices.keyboard.key(
            device_id="deviceId",
            key=0,
            x_device_display_id=0,
        )
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_key(self, client: Mobilerun) -> None:
        response = client.devices.keyboard.with_raw_response.key(
            device_id="deviceId",
            key=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        keyboard = response.parse()
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_key(self, client: Mobilerun) -> None:
        with client.devices.keyboard.with_streaming_response.key(
            device_id="deviceId",
            key=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            keyboard = response.parse()
            assert keyboard is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_key(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.keyboard.with_raw_response.key(
                device_id="",
                key=0,
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_write(self, client: Mobilerun) -> None:
        keyboard = client.devices.keyboard.write(
            device_id="deviceId",
            text="text",
        )
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_write_with_all_params(self, client: Mobilerun) -> None:
        keyboard = client.devices.keyboard.write(
            device_id="deviceId",
            text="text",
            clear=True,
            stealth=True,
            wpm=0,
            x_device_display_id=0,
        )
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_write(self, client: Mobilerun) -> None:
        response = client.devices.keyboard.with_raw_response.write(
            device_id="deviceId",
            text="text",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        keyboard = response.parse()
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_write(self, client: Mobilerun) -> None:
        with client.devices.keyboard.with_streaming_response.write(
            device_id="deviceId",
            text="text",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            keyboard = response.parse()
            assert keyboard is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_write(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.keyboard.with_raw_response.write(
                device_id="",
                text="text",
            )


class TestAsyncKeyboard:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_clear(self, async_client: AsyncMobilerun) -> None:
        keyboard = await async_client.devices.keyboard.clear(
            device_id="deviceId",
        )
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_clear_with_all_params(self, async_client: AsyncMobilerun) -> None:
        keyboard = await async_client.devices.keyboard.clear(
            device_id="deviceId",
            x_device_display_id=0,
        )
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_clear(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.keyboard.with_raw_response.clear(
            device_id="deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        keyboard = await response.parse()
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_clear(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.keyboard.with_streaming_response.clear(
            device_id="deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            keyboard = await response.parse()
            assert keyboard is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_clear(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.keyboard.with_raw_response.clear(
                device_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_key(self, async_client: AsyncMobilerun) -> None:
        keyboard = await async_client.devices.keyboard.key(
            device_id="deviceId",
            key=0,
        )
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_key_with_all_params(self, async_client: AsyncMobilerun) -> None:
        keyboard = await async_client.devices.keyboard.key(
            device_id="deviceId",
            key=0,
            x_device_display_id=0,
        )
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_key(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.keyboard.with_raw_response.key(
            device_id="deviceId",
            key=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        keyboard = await response.parse()
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_key(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.keyboard.with_streaming_response.key(
            device_id="deviceId",
            key=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            keyboard = await response.parse()
            assert keyboard is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_key(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.keyboard.with_raw_response.key(
                device_id="",
                key=0,
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_write(self, async_client: AsyncMobilerun) -> None:
        keyboard = await async_client.devices.keyboard.write(
            device_id="deviceId",
            text="text",
        )
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_write_with_all_params(self, async_client: AsyncMobilerun) -> None:
        keyboard = await async_client.devices.keyboard.write(
            device_id="deviceId",
            text="text",
            clear=True,
            stealth=True,
            wpm=0,
            x_device_display_id=0,
        )
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_write(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.keyboard.with_raw_response.write(
            device_id="deviceId",
            text="text",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        keyboard = await response.parse()
        assert keyboard is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_write(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.keyboard.with_streaming_response.write(
            device_id="deviceId",
            text="text",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            keyboard = await response.parse()
            assert keyboard is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_write(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.keyboard.with_raw_response.write(
                device_id="",
                text="text",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Optional, cast

import pytest

from mobilerun import Mobilerun, AsyncMobilerun
from tests.utils import assert_matches_type
from mobilerun.types.devices import AppListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestApps:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Mobilerun) -> None:
        app = client.devices.apps.update(
            package_name="packageName",
            device_id="deviceId",
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Mobilerun) -> None:
        app = client.devices.apps.update(
            package_name="packageName",
            device_id="deviceId",
            x_device_display_id=0,
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Mobilerun) -> None:
        response = client.devices.apps.with_raw_response.update(
            package_name="packageName",
            device_id="deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app = response.parse()
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Mobilerun) -> None:
        with client.devices.apps.with_streaming_response.update(
            package_name="packageName",
            device_id="deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app = response.parse()
            assert app is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.apps.with_raw_response.update(
                package_name="packageName",
                device_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            client.devices.apps.with_raw_response.update(
                package_name="",
                device_id="deviceId",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Mobilerun) -> None:
        app = client.devices.apps.list(
            device_id="deviceId",
        )
        assert_matches_type(Optional[AppListResponse], app, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Mobilerun) -> None:
        app = client.devices.apps.list(
            device_id="deviceId",
            include_protected_apps=True,
            include_system_apps=True,
            x_device_display_id=0,
        )
        assert_matches_type(Optional[AppListResponse], app, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Mobilerun) -> None:
        response = client.devices.apps.with_raw_response.list(
            device_id="deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app = response.parse()
        assert_matches_type(Optional[AppListResponse], app, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Mobilerun) -> None:
        with client.devices.apps.with_streaming_response.list(
            device_id="deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app = response.parse()
            assert_matches_type(Optional[AppListResponse], app, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.apps.with_raw_response.list(
                device_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Mobilerun) -> None:
        app = client.devices.apps.delete(
            package_name="packageName",
            device_id="deviceId",
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete_with_all_params(self, client: Mobilerun) -> None:
        app = client.devices.apps.delete(
            package_name="packageName",
            device_id="deviceId",
            x_device_display_id=0,
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Mobilerun) -> None:
        response = client.devices.apps.with_raw_response.delete(
            package_name="packageName",
            device_id="deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app = response.parse()
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Mobilerun) -> None:
        with client.devices.apps.with_streaming_response.delete(
            package_name="packageName",
            device_id="deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app = response.parse()
            assert app is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.apps.with_raw_response.delete(
                package_name="packageName",
                device_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            client.devices.apps.with_raw_response.delete(
                package_name="",
                device_id="deviceId",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_install(self, client: Mobilerun) -> None:
        app = client.devices.apps.install(
            device_id="deviceId",
            package_name="packageName",
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_install_with_all_params(self, client: Mobilerun) -> None:
        app = client.devices.apps.install(
            device_id="deviceId",
            package_name="packageName",
            x_device_display_id=0,
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_install(self, client: Mobilerun) -> None:
        response = client.devices.apps.with_raw_response.install(
            device_id="deviceId",
            package_name="packageName",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app = response.parse()
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_install(self, client: Mobilerun) -> None:
        with client.devices.apps.with_streaming_response.install(
            device_id="deviceId",
            package_name="packageName",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app = response.parse()
            assert app is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_install(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.apps.with_raw_response.install(
                device_id="",
                package_name="packageName",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_start(self, client: Mobilerun) -> None:
        app = client.devices.apps.start(
            package_name="packageName",
            device_id="deviceId",
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_start_with_all_params(self, client: Mobilerun) -> None:
        app = client.devices.apps.start(
            package_name="packageName",
            device_id="deviceId",
            activity="activity",
            x_device_display_id=0,
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_start(self, client: Mobilerun) -> None:
        response = client.devices.apps.with_raw_response.start(
            package_name="packageName",
            device_id="deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app = response.parse()
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_start(self, client: Mobilerun) -> None:
        with client.devices.apps.with_streaming_response.start(
            package_name="packageName",
            device_id="deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app = response.parse()
            assert app is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_start(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.apps.with_raw_response.start(
                package_name="packageName",
                device_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            client.devices.apps.with_raw_response.start(
                package_name="",
                device_id="deviceId",
            )


class TestAsyncApps:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncMobilerun) -> None:
        app = await async_client.devices.apps.update(
            package_name="packageName",
            device_id="deviceId",
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncMobilerun) -> None:
        app = await async_client.devices.apps.update(
            package_name="packageName",
            device_id="deviceId",
            x_device_display_id=0,
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.apps.with_raw_response.update(
            package_name="packageName",
            device_id="deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app = await response.parse()
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.apps.with_streaming_response.update(
            package_name="packageName",
            device_id="deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app = await response.parse()
            assert app is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.apps.with_raw_response.update(
                package_name="packageName",
                device_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            await async_client.devices.apps.with_raw_response.update(
                package_name="",
                device_id="deviceId",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncMobilerun) -> None:
        app = await async_client.devices.apps.list(
            device_id="deviceId",
        )
        assert_matches_type(Optional[AppListResponse], app, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncMobilerun) -> None:
        app = await async_client.devices.apps.list(
            device_id="deviceId",
            include_protected_apps=True,
            include_system_apps=True,
            x_device_display_id=0,
        )
        assert_matches_type(Optional[AppListResponse], app, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.apps.with_raw_response.list(
            device_id="deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app = await response.parse()
        assert_matches_type(Optional[AppListResponse], app, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.apps.with_streaming_response.list(
            device_id="deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app = await response.parse()
            assert_matches_type(Optional[AppListResponse], app, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.apps.with_raw_response.list(
                device_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncMobilerun) -> None:
        app = await async_client.devices.apps.delete(
            package_name="packageName",
            device_id="deviceId",
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete_with_all_params(self, async_client: AsyncMobilerun) -> None:
        app = await async_client.devices.apps.delete(
            package_name="packageName",
            device_id="deviceId",
            x_device_display_id=0,
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.apps.with_raw_response.delete(
            package_name="packageName",
            device_id="deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app = await response.parse()
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.apps.with_streaming_response.delete(
            package_name="packageName",
            device_id="deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app = await response.parse()
            assert app is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.apps.with_raw_response.delete(
                package_name="packageName",
                device_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            await async_client.devices.apps.with_raw_response.delete(
                package_name="",
                device_id="deviceId",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_install(self, async_client: AsyncMobilerun) -> None:
        app = await async_client.devices.apps.install(
            device_id="deviceId",
            package_name="packageName",
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_install_with_all_params(self, async_client: AsyncMobilerun) -> None:
        app = await async_client.devices.apps.install(
            device_id="deviceId",
            package_name="packageName",
            x_device_display_id=0,
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_install(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.apps.with_raw_response.install(
            device_id="deviceId",
            package_name="packageName",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app = await response.parse()
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_install(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.apps.with_streaming_response.install(
            device_id="deviceId",
            package_name="packageName",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app = await response.parse()
            assert app is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_install(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.apps.with_raw_response.install(
                device_id="",
                package_name="packageName",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_start(self, async_client: AsyncMobilerun) -> None:
        app = await async_client.devices.apps.start(
            package_name="packageName",
            device_id="deviceId",
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_start_with_all_params(self, async_client: AsyncMobilerun) -> None:
        app = await async_client.devices.apps.start(
            package_name="packageName",
            device_id="deviceId",
            activity="activity",
            x_device_display_id=0,
        )
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_start(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.apps.with_raw_response.start(
            package_name="packageName",
            device_id="deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app = await response.parse()
        assert app is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_start(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.apps.with_streaming_response.start(
            package_name="packageName",
            device_id="deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app = await response.parse()
            assert app is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_start(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.apps.with_raw_response.start(
                package_name="packageName",
                device_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `package_name` but received ''"):
            await async_client.devices.apps.with_raw_response.start(
                package_name="",
                device_id="deviceId",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["CarrierDeleteResponse"]


class CarrierDeleteResponse(BaseModel):
    message: str

    schema_: Optional[str] = FieldInfo(alias="$schema", default=None)
    """A URL to the JSON Schema for this object."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .packages.credential import Credential

__all__ = ["PackageListResponse"]


class PackageListResponse(BaseModel):
    data: List[Credential]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["TaskSendMessageParams"]


class TaskSendMessageParams(TypedDict, total=False):
    message: Required[str]
    """Message to send to the running agent"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from mobilerun import Mobilerun, AsyncMobilerun
from tests.utils import assert_matches_type
from mobilerun.types import (
    HookListResponse,
    HookUpdateResponse,
    HookPerformResponse,
    HookRetrieveResponse,
    HookSubscribeResponse,
    HookUnsubscribeResponse,
    HookGetSampleDataResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestHooks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Mobilerun) -> None:
        hook = client.hooks.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(HookRetrieveResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Mobilerun) -> None:
        response = client.hooks.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        hook = response.parse()
        assert_matches_type(HookRetrieveResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Mobilerun) -> None:
        with client.hooks.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            hook = response.parse()
            assert_matches_type(HookRetrieveResponse, hook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `hook_id` but received ''"):
            client.hooks.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Mobilerun) -> None:
        hook = client.hooks.update(
            hook_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(HookUpdateResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Mobilerun) -> None:
        hook = client.hooks.update(
            hook_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            events=["string"],
            state="state",
        )
        assert_matches_type(HookUpdateResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Mobilerun) -> None:
        response = client.hooks.with_raw_response.update(
            hook_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        hook = response.parse()
        assert_matches_type(HookUpdateResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Mobilerun) -> None:
        with client.hooks.with_streaming_response.update(
            hook_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            hook = response.parse()
            assert_matches_type(HookUpdateResponse, hook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `hook_id` but received ''"):
            client.hooks.with_raw_response.update(
                hook_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Mobilerun) -> None:
        hook = client.hooks.list()
        assert_matches_type(HookListResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Mobilerun) -> None:
        hook = client.hooks.list(
            order_by="orderBy",
            order_by_direction="asc",
            page=1,
            page_size=1,
        )
        assert_matches_type(HookListResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Mobilerun) -> None:
        response = client.hooks.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        hook = response.parse()
        assert_matches_type(HookListResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Mobilerun) -> None:
        with client.hooks.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            hook = response.parse()
            assert_matches_type(HookListResponse, hook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_sample_data(self, client: Mobilerun) -> None:
        hook = client.hooks.get_sample_data()
        assert_matches_type(HookGetSampleDataResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_sample_data(self, client: Mobilerun) -> None:
        response = client.hooks.with_raw_response.get_sample_data()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        hook = response.parse()
        assert_matches_type(HookGetSampleDataResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_sample_data(self, client: Mobilerun) -> None:
        with client.hooks.with_streaming_response.get_sample_data() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            hook = response.parse()
            assert_matches_type(HookGetSampleDataResponse, hook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_perform(self, client: Mobilerun) -> None:
        hook = client.hooks.perform(
            body={"foo": "bar"},
        )
        assert_matches_type(HookPerformResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_perform(self, client: Mobilerun) -> None:
        response = client.hooks.with_raw_response.perform(
            body={"foo": "bar"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        hook = response.parse()
        assert_matches_type(HookPerformResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_perform(self, client: Mobilerun) -> None:
        with client.hooks.with_streaming_response.perform(
            body={"foo": "bar"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            hook = response.parse()
            assert_matches_type(HookPerformResponse, hook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_subscribe(self, client: Mobilerun) -> None:
        hook = client.hooks.subscribe(
            target_url="https://example.com",
        )
        assert_matches_type(HookSubscribeResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_subscribe_with_all_params(self, client: Mobilerun) -> None:
        hook = client.hooks.subscribe(
            target_url="https://example.com",
            events=["string"],
            service="service",
        )
        assert_matches_type(HookSubscribeResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_subscribe(self, client: Mobilerun) -> None:
        response = client.hooks.with_raw_response.subscribe(
            target_url="https://example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        hook = response.parse()
        assert_matches_type(HookSubscribeResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_subscribe(self, client: Mobilerun) -> None:
        with client.hooks.with_streaming_response.subscribe(
            target_url="https://example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            hook = response.parse()
            assert_matches_type(HookSubscribeResponse, hook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_unsubscribe(self, client: Mobilerun) -> None:
        hook = client.hooks.unsubscribe(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(HookUnsubscribeResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_unsubscribe(self, client: Mobilerun) -> None:
        response = client.hooks.with_raw_response.unsubscribe(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        hook = response.parse()
        assert_matches_type(HookUnsubscribeResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_unsubscribe(self, client: Mobilerun) -> None:
        with client.hooks.with_streaming_response.unsubscribe(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            hook = response.parse()
            assert_matches_type(HookUnsubscribeResponse, hook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_unsubscribe(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `hook_id` but received ''"):
            client.hooks.with_raw_response.unsubscribe(
                "",
            )


class TestAsyncHooks:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncMobilerun) -> None:
        hook = await async_client.hooks.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(HookRetrieveResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.hooks.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        hook = await response.parse()
        assert_matches_type(HookRetrieveResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncMobilerun) -> None:
        async with async_client.hooks.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            hook = await response.parse()
            assert_matches_type(HookRetrieveResponse, hook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `hook_id` but received ''"):
            await async_client.hooks.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncMobilerun) -> None:
        hook = await async_client.hooks.update(
            hook_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(HookUpdateResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncMobilerun) -> None:
        hook = await async_client.hooks.update(
            hook_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            events=["string"],
            state="state",
        )
        assert_matches_type(HookUpdateResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.hooks.with_raw_response.update(
            hook_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        hook = await response.parse()
        assert_matches_type(HookUpdateResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncMobilerun) -> None:
        async with async_client.hooks.with_streaming_response.update(
            hook_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            hook = await response.parse()
            assert_matches_type(HookUpdateResponse, hook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `hook_id` but received ''"):
            await async_client.hooks.with_raw_response.update(
                hook_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncMobilerun) -> None:
        hook = await async_client.hooks.list()
        assert_matches_type(HookListResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncMobilerun) -> None:
        hook = await async_client.hooks.list(
            order_by="orderBy",
            order_by_direction="asc",
            page=1,
            page_size=1,
        )
        assert_matches_type(HookListResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.hooks.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        hook = await response.parse()
        assert_matches_type(HookListResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncMobilerun) -> None:
        async with async_client.hooks.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            hook = await response.parse()
            assert_matches_type(HookListResponse, hook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_sample_data(self, async_client: AsyncMobilerun) -> None:
        hook = await async_client.hooks.get_sample_data()
        assert_matches_type(HookGetSampleDataResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_sample_data(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.hooks.with_raw_response.get_sample_data()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        hook = await response.parse()
        assert_matches_type(HookGetSampleDataResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_sample_data(self, async_client: AsyncMobilerun) -> None:
        async with async_client.hooks.with_streaming_response.get_sample_data() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            hook = await response.parse()
            assert_matches_type(HookGetSampleDataResponse, hook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_perform(self, async_client: AsyncMobilerun) -> None:
        hook = await async_client.hooks.perform(
            body={"foo": "bar"},
        )
        assert_matches_type(HookPerformResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_perform(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.hooks.with_raw_response.perform(
            body={"foo": "bar"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        hook = await response.parse()
        assert_matches_type(HookPerformResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_perform(self, async_client: AsyncMobilerun) -> None:
        async with async_client.hooks.with_streaming_response.perform(
            body={"foo": "bar"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            hook = await response.parse()
            assert_matches_type(HookPerformResponse, hook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_subscribe(self, async_client: AsyncMobilerun) -> None:
        hook = await async_client.hooks.subscribe(
            target_url="https://example.com",
        )
        assert_matches_type(HookSubscribeResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_subscribe_with_all_params(self, async_client: AsyncMobilerun) -> None:
        hook = await async_client.hooks.subscribe(
            target_url="https://example.com",
            events=["string"],
            service="service",
        )
        assert_matches_type(HookSubscribeResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_subscribe(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.hooks.with_raw_response.subscribe(
            target_url="https://example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        hook = await response.parse()
        assert_matches_type(HookSubscribeResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_subscribe(self, async_client: AsyncMobilerun) -> None:
        async with async_client.hooks.with_streaming_response.subscribe(
            target_url="https://example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            hook = await response.parse()
            assert_matches_type(HookSubscribeResponse, hook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_unsubscribe(self, async_client: AsyncMobilerun) -> None:
        hook = await async_client.hooks.unsubscribe(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(HookUnsubscribeResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_unsubscribe(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.hooks.with_raw_response.unsubscribe(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        hook = await response.parse()
        assert_matches_type(HookUnsubscribeResponse, hook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_unsubscribe(self, async_client: AsyncMobilerun) -> None:
        async with async_client.hooks.with_streaming_response.unsubscribe(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            hook = await response.parse()
            assert_matches_type(HookUnsubscribeResponse, hook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_unsubscribe(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `hook_id` but received ''"):
            await async_client.hooks.with_raw_response.unsubscribe(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.credentials import package_create_params
from .credentials.credentials import (
    CredentialsResource,
    AsyncCredentialsResource,
    CredentialsResourceWithRawResponse,
    AsyncCredentialsResourceWithRawResponse,
    CredentialsResourceWithStreamingResponse,
    AsyncCredentialsResourceWithStreamingResponse,
)
from ....types.credentials.package_list_response import PackageListResponse
from ....types.credentials.package_create_response import PackageCreateResponse

__all__ = ["PackagesResource", "AsyncPackagesResource"]


class PackagesResource(SyncAPIResource):
    @cached_property
    def credentials(self) -> CredentialsResource:
        return CredentialsResource(self._client)

    @cached_property
    def with_raw_response(self) -> PackagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#accessing-raw-response-data-eg-headers
        """
        return PackagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PackagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#with_streaming_response
        """
        return PackagesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        package_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PackageCreateResponse:
        """
        Initialize a new package/app

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/credentials/packages",
            body=maybe_transform({"package_name": package_name}, package_create_params.PackageCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PackageCreateResponse,
        )

    def list(
        self,
        package_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PackageListResponse:
        """
        List credentials for a specific package

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        return self._get(
            f"/credentials/packages/{package_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PackageListResponse,
        )


class AsyncPackagesResource(AsyncAPIResource):
    @cached_property
    def credentials(self) -> AsyncCredentialsResource:
        return AsyncCredentialsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPackagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPackagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPackagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#with_streaming_response
        """
        return AsyncPackagesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        package_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PackageCreateResponse:
        """
        Initialize a new package/app

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/credentials/packages",
            body=await async_maybe_transform({"package_name": package_name}, package_create_params.PackageCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PackageCreateResponse,
        )

    async def list(
        self,
        package_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PackageListResponse:
        """
        List credentials for a specific package

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        return await self._get(
            f"/credentials/packages/{package_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PackageListResponse,
        )


class PackagesResourceWithRawResponse:
    def __init__(self, packages: PackagesResource) -> None:
        self._packages = packages

        self.create = to_raw_response_wrapper(
            packages.create,
        )
        self.list = to_raw_response_wrapper(
            packages.list,
        )

    @cached_property
    def credentials(self) -> CredentialsResourceWithRawResponse:
        return CredentialsResourceWithRawResponse(self._packages.credentials)


class AsyncPackagesResourceWithRawResponse:
    def __init__(self, packages: AsyncPackagesResource) -> None:
        self._packages = packages

        self.create = async_to_raw_response_wrapper(
            packages.create,
        )
        self.list = async_to_raw_response_wrapper(
            packages.list,
        )

    @cached_property
    def credentials(self) -> AsyncCredentialsResourceWithRawResponse:
        return AsyncCredentialsResourceWithRawResponse(self._packages.credentials)


class PackagesResourceWithStreamingResponse:
    def __init__(self, packages: PackagesResource) -> None:
        self._packages = packages

        self.create = to_streamed_response_wrapper(
            packages.create,
        )
        self.list = to_streamed_response_wrapper(
            packages.list,
        )

    @cached_property
    def credentials(self) -> CredentialsResourceWithStreamingResponse:
        return CredentialsResourceWithStreamingResponse(self._packages.credentials)


class AsyncPackagesResourceWithStreamingResponse:
    def __init__(self, packages: AsyncPackagesResource) -> None:
        self._packages = packages

        self.create = async_to_streamed_response_wrapper(
            packages.create,
        )
        self.list = async_to_streamed_response_wrapper(
            packages.list,
        )

    @cached_property
    def credentials(self) -> AsyncCredentialsResourceWithStreamingResponse:
        return AsyncCredentialsResourceWithStreamingResponse(self._packages.credentials)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .credential import Credential as Credential
from .credential_create_params import CredentialCreateParams as CredentialCreateParams
from .credential_create_response import CredentialCreateResponse as CredentialCreateResponse
from .credential_delete_response import CredentialDeleteResponse as CredentialDeleteResponse
from .credential_retrieve_response import CredentialRetrieveResponse as CredentialRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal

import httpx

from ...types import TaskStatus, task_run_params, task_list_params, task_run_streamed_params, task_send_message_params
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .ui_states import (
    UiStatesResource,
    AsyncUiStatesResource,
    UiStatesResourceWithRawResponse,
    AsyncUiStatesResourceWithRawResponse,
    UiStatesResourceWithStreamingResponse,
    AsyncUiStatesResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .screenshots import (
    ScreenshotsResource,
    AsyncScreenshotsResource,
    ScreenshotsResourceWithRawResponse,
    AsyncScreenshotsResourceWithRawResponse,
    ScreenshotsResourceWithStreamingResponse,
    AsyncScreenshotsResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from ...types.task_status import TaskStatus
from ...types.task_run_response import TaskRunResponse
from ...types.task_list_response import TaskListResponse
from ...types.task_stop_response import TaskStopResponse
from ...types.task_retrieve_response import TaskRetrieveResponse
from ...types.task_get_status_response import TaskGetStatusResponse
from ...types.package_credentials_param import PackageCredentialsParam
from ...types.task_send_message_response import TaskSendMessageResponse
from ...types.task_get_trajectory_response import TaskGetTrajectoryResponse

__all__ = ["TasksResource", "AsyncTasksResource"]


class TasksResource(SyncAPIResource):
    """Tasks API"""

    @cached_property
    def screenshots(self) -> ScreenshotsResource:
        """Tasks API"""
        return ScreenshotsResource(self._client)

    @cached_property
    def ui_states(self) -> UiStatesResource:
        """Tasks API"""
        return UiStatesResource(self._client)

    @cached_property
    def with_raw_response(self) -> TasksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#accessing-raw-response-data-eg-headers
        """
        return TasksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TasksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#with_streaming_response
        """
        return TasksResourceWithStreamingResponse(self)

    def retrieve(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskRetrieveResponse:
        """
        Get full details of a task by ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return self._get(
            f"/tasks/{task_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskRetrieveResponse,
        )

    def list(
        self,
        *,
        order_by: Optional[Literal["id", "createdAt", "finishedAt", "status"]] | Omit = omit,
        order_by_direction: Literal["asc", "desc"] | Omit = omit,
        page: int | Omit = omit,
        page_size: int | Omit = omit,
        query: Optional[str] | Omit = omit,
        status: Optional[TaskStatus] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskListResponse:
        """
        List tasks with optional filtering, sorting, and pagination.

        Args:
          query: Search in task description.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/tasks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "order_by": order_by,
                        "order_by_direction": order_by_direction,
                        "page": page,
                        "page_size": page_size,
                        "query": query,
                        "status": status,
                    },
                    task_list_params.TaskListParams,
                ),
            ),
            cast_to=TaskListResponse,
        )

    def attach(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Attach to a running task and receive its events as an SSE stream.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            f"/tasks/{task_id}/attach",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get_status(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskGetStatusResponse:
        """
        Get the status of a task.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return self._get(
            f"/tasks/{task_id}/status",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskGetStatusResponse,
        )

    def get_trajectory(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskGetTrajectoryResponse:
        """
        Get the trajectory of a task.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return self._get(
            f"/tasks/{task_id}/trajectory",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskGetTrajectoryResponse,
        )

    def run(
        self,
        *,
        device_id: str,
        task: str,
        agent_id: int | Omit = omit,
        apps: SequenceNotStr[str] | Omit = omit,
        credentials: Iterable[PackageCredentialsParam] | Omit = omit,
        display_id: int | Omit = omit,
        execution_timeout: int | Omit = omit,
        files: SequenceNotStr[str] | Omit = omit,
        llm_model: str | Omit = omit,
        max_steps: int | Omit = omit,
        output_schema: Optional[Dict[str, object]] | Omit = omit,
        reasoning: bool | Omit = omit,
        stealth: bool | Omit = omit,
        temperature: float | Omit = omit,
        vision: bool | Omit = omit,
        vpn_country: Optional[Literal["US", "BR", "FR", "DE", "IN", "JP", "KR", "ZA"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskRunResponse:
        """Create and dispatch a new agent task.

        Returns the task ID and device stream
        details.

        Args:
          device_id: The ID of the device to run the task on.

          display_id: The display ID of the device to run the task on.

          llm_model: The LLM model identifier to use for the task (e.g.
              'google/gemini-3.1-flash-lite-preview')

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/tasks",
            body=maybe_transform(
                {
                    "device_id": device_id,
                    "task": task,
                    "agent_id": agent_id,
                    "apps": apps,
                    "credentials": credentials,
                    "display_id": display_id,
                    "execution_timeout": execution_timeout,
                    "files": files,
                    "llm_model": llm_model,
                    "max_steps": max_steps,
                    "output_schema": output_schema,
                    "reasoning": reasoning,
                    "stealth": stealth,
                    "temperature": temperature,
                    "vision": vision,
                    "vpn_country": vpn_country,
                },
                task_run_params.TaskRunParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskRunResponse,
        )

    def run_streamed(
        self,
        *,
        device_id: str,
        task: str,
        agent_id: int | Omit = omit,
        apps: SequenceNotStr[str] | Omit = omit,
        credentials: Iterable[PackageCredentialsParam] | Omit = omit,
        display_id: int | Omit = omit,
        execution_timeout: int | Omit = omit,
        files: SequenceNotStr[str] | Omit = omit,
        llm_model: str | Omit = omit,
        max_steps: int | Omit = omit,
        output_schema: Optional[Dict[str, object]] | Omit = omit,
        reasoning: bool | Omit = omit,
        stealth: bool | Omit = omit,
        temperature: float | Omit = omit,
        vision: bool | Omit = omit,
        vpn_country: Optional[Literal["US", "BR", "FR", "DE", "IN", "JP", "KR", "ZA"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Create and dispatch a new agent task, returning an SSE stream of task events.
        Cancels the task if the client disconnects.

        Args:
          device_id: The ID of the device to run the task on.

          display_id: The display ID of the device to run the task on.

          llm_model: The LLM model identifier to use for the task (e.g.
              'google/gemini-3.1-flash-lite-preview')

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/tasks/stream",
            body=maybe_transform(
                {
                    "device_id": device_id,
                    "task": task,
                    "agent_id": agent_id,
                    "apps": apps,
                    "credentials": credentials,
                    "display_id": display_id,
                    "execution_timeout": execution_timeout,
                    "files": files,
                    "llm_model": llm_model,
                    "max_steps": max_steps,
                    "output_schema": output_schema,
                    "reasoning": reasoning,
                    "stealth": stealth,
                    "temperature": temperature,
                    "vision": vision,
                    "vpn_country": vpn_country,
                },
                task_run_streamed_params.TaskRunStreamedParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def send_message(
        self,
        task_id: str,
        *,
        message: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskSendMessageResponse:
        """Send a message to a running agent task.

        The message ID is delivered via SSE
        (UserMessageEvent with action=queued).

        Args:
          message: Message to send to the running agent

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return self._post(
            f"/tasks/{task_id}/message",
            body=maybe_transform({"message": message}, task_send_message_params.TaskSendMessageParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskSendMessageResponse,
        )

    def stop(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskStopResponse:
        """Cancel a running task.

        Returns an error if the task is already in a terminal
        state.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return self._post(
            f"/tasks/{task_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskStopResponse,
        )


class AsyncTasksResource(AsyncAPIResource):
    """Tasks API"""

    @cached_property
    def screenshots(self) -> AsyncScreenshotsResource:
        """Tasks API"""
        return AsyncScreenshotsResource(self._client)

    @cached_property
    def ui_states(self) -> AsyncUiStatesResource:
        """Tasks API"""
        return AsyncUiStatesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTasksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTasksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTasksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#with_streaming_response
        """
        return AsyncTasksResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskRetrieveResponse:
        """
        Get full details of a task by ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return await self._get(
            f"/tasks/{task_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskRetrieveResponse,
        )

    async def list(
        self,
        *,
        order_by: Optional[Literal["id", "createdAt", "finishedAt", "status"]] | Omit = omit,
        order_by_direction: Literal["asc", "desc"] | Omit = omit,
        page: int | Omit = omit,
        page_size: int | Omit = omit,
        query: Optional[str] | Omit = omit,
        status: Optional[TaskStatus] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskListResponse:
        """
        List tasks with optional filtering, sorting, and pagination.

        Args:
          query: Search in task description.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/tasks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "order_by": order_by,
                        "order_by_direction": order_by_direction,
                        "page": page,
                        "page_size": page_size,
                        "query": query,
                        "status": status,
                    },
                    task_list_params.TaskListParams,
                ),
            ),
            cast_to=TaskListResponse,
        )

    async def attach(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Attach to a running task and receive its events as an SSE stream.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            f"/tasks/{task_id}/attach",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get_status(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskGetStatusResponse:
        """
        Get the status of a task.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return await self._get(
            f"/tasks/{task_id}/status",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskGetStatusResponse,
        )

    async def get_trajectory(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskGetTrajectoryResponse:
        """
        Get the trajectory of a task.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return await self._get(
            f"/tasks/{task_id}/trajectory",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskGetTrajectoryResponse,
        )

    async def run(
        self,
        *,
        device_id: str,
        task: str,
        agent_id: int | Omit = omit,
        apps: SequenceNotStr[str] | Omit = omit,
        credentials: Iterable[PackageCredentialsParam] | Omit = omit,
        display_id: int | Omit = omit,
        execution_timeout: int | Omit = omit,
        files: SequenceNotStr[str] | Omit = omit,
        llm_model: str | Omit = omit,
        max_steps: int | Omit = omit,
        output_schema: Optional[Dict[str, object]] | Omit = omit,
        reasoning: bool | Omit = omit,
        stealth: bool | Omit = omit,
        temperature: float | Omit = omit,
        vision: bool | Omit = omit,
        vpn_country: Optional[Literal["US", "BR", "FR", "DE", "IN", "JP", "KR", "ZA"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskRunResponse:
        """Create and dispatch a new agent task.

        Returns the task ID and device stream
        details.

        Args:
          device_id: The ID of the device to run the task on.

          display_id: The display ID of the device to run the task on.

          llm_model: The LLM model identifier to use for the task (e.g.
              'google/gemini-3.1-flash-lite-preview')

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/tasks",
            body=await async_maybe_transform(
                {
                    "device_id": device_id,
                    "task": task,
                    "agent_id": agent_id,
                    "apps": apps,
                    "credentials": credentials,
                    "display_id": display_id,
                    "execution_timeout": execution_timeout,
                    "files": files,
                    "llm_model": llm_model,
                    "max_steps": max_steps,
                    "output_schema": output_schema,
                    "reasoning": reasoning,
                    "stealth": stealth,
                    "temperature": temperature,
                    "vision": vision,
                    "vpn_country": vpn_country,
                },
                task_run_params.TaskRunParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskRunResponse,
        )

    async def run_streamed(
        self,
        *,
        device_id: str,
        task: str,
        agent_id: int | Omit = omit,
        apps: SequenceNotStr[str] | Omit = omit,
        credentials: Iterable[PackageCredentialsParam] | Omit = omit,
        display_id: int | Omit = omit,
        execution_timeout: int | Omit = omit,
        files: SequenceNotStr[str] | Omit = omit,
        llm_model: str | Omit = omit,
        max_steps: int | Omit = omit,
        output_schema: Optional[Dict[str, object]] | Omit = omit,
        reasoning: bool | Omit = omit,
        stealth: bool | Omit = omit,
        temperature: float | Omit = omit,
        vision: bool | Omit = omit,
        vpn_country: Optional[Literal["US", "BR", "FR", "DE", "IN", "JP", "KR", "ZA"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Create and dispatch a new agent task, returning an SSE stream of task events.
        Cancels the task if the client disconnects.

        Args:
          device_id: The ID of the device to run the task on.

          display_id: The display ID of the device to run the task on.

          llm_model: The LLM model identifier to use for the task (e.g.
              'google/gemini-3.1-flash-lite-preview')

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/tasks/stream",
            body=await async_maybe_transform(
                {
                    "device_id": device_id,
                    "task": task,
                    "agent_id": agent_id,
                    "apps": apps,
                    "credentials": credentials,
                    "display_id": display_id,
                    "execution_timeout": execution_timeout,
                    "files": files,
                    "llm_model": llm_model,
                    "max_steps": max_steps,
                    "output_schema": output_schema,
                    "reasoning": reasoning,
                    "stealth": stealth,
                    "temperature": temperature,
                    "vision": vision,
                    "vpn_country": vpn_country,
                },
                task_run_streamed_params.TaskRunStreamedParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def send_message(
        self,
        task_id: str,
        *,
        message: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskSendMessageResponse:
        """Send a message to a running agent task.

        The message ID is delivered via SSE
        (UserMessageEvent with action=queued).

        Args:
          message: Message to send to the running agent

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return await self._post(
            f"/tasks/{task_id}/message",
            body=await async_maybe_transform({"message": message}, task_send_message_params.TaskSendMessageParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskSendMessageResponse,
        )

    async def stop(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskStopResponse:
        """Cancel a running task.

        Returns an error if the task is already in a terminal
        state.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return await self._post(
            f"/tasks/{task_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskStopResponse,
        )


class TasksResourceWithRawResponse:
    def __init__(self, tasks: TasksResource) -> None:
        self._tasks = tasks

        self.retrieve = to_raw_response_wrapper(
            tasks.retrieve,
        )
        self.list = to_raw_response_wrapper(
            tasks.list,
        )
        self.attach = to_raw_response_wrapper(
            tasks.attach,
        )
        self.get_status = to_raw_response_wrapper(
            tasks.get_status,
        )
        self.get_trajectory = to_raw_response_wrapper(
            tasks.get_trajectory,
        )
        self.run = to_raw_response_wrapper(
            tasks.run,
        )
        self.run_streamed = to_raw_response_wrapper(
            tasks.run_streamed,
        )
        self.send_message = to_raw_response_wrapper(
            tasks.send_message,
        )
        self.stop = to_raw_response_wrapper(
            tasks.stop,
        )

    @cached_property
    def screenshots(self) -> ScreenshotsResourceWithRawResponse:
        """Tasks API"""
        return ScreenshotsResourceWithRawResponse(self._tasks.screenshots)

    @cached_property
    def ui_states(self) -> UiStatesResourceWithRawResponse:
        """Tasks API"""
        return UiStatesResourceWithRawResponse(self._tasks.ui_states)


class AsyncTasksResourceWithRawResponse:
    def __init__(self, tasks: AsyncTasksResource) -> None:
        self._tasks = tasks

        self.retrieve = async_to_raw_response_wrapper(
            tasks.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            tasks.list,
        )
        self.attach = async_to_raw_response_wrapper(
            tasks.attach,
        )
        self.get_status = async_to_raw_response_wrapper(
            tasks.get_status,
        )
        self.get_trajectory = async_to_raw_response_wrapper(
            tasks.get_trajectory,
        )
        self.run = async_to_raw_response_wrapper(
            tasks.run,
        )
        self.run_streamed = async_to_raw_response_wrapper(
            tasks.run_streamed,
        )
        self.send_message = async_to_raw_response_wrapper(
            tasks.send_message,
        )
        self.stop = async_to_raw_response_wrapper(
            tasks.stop,
        )

    @cached_property
    def screenshots(self) -> AsyncScreenshotsResourceWithRawResponse:
        """Tasks API"""
        return AsyncScreenshotsResourceWithRawResponse(self._tasks.screenshots)

    @cached_property
    def ui_states(self) -> AsyncUiStatesResourceWithRawResponse:
        """Tasks API"""
        return AsyncUiStatesResourceWithRawResponse(self._tasks.ui_states)


class TasksResourceWithStreamingResponse:
    def __init__(self, tasks: TasksResource) -> None:
        self._tasks = tasks

        self.retrieve = to_streamed_response_wrapper(
            tasks.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            tasks.list,
        )
        self.attach = to_streamed_response_wrapper(
            tasks.attach,
        )
        self.get_status = to_streamed_response_wrapper(
            tasks.get_status,
        )
        self.get_trajectory = to_streamed_response_wrapper(
            tasks.get_trajectory,
        )
        self.run = to_streamed_response_wrapper(
            tasks.run,
        )
        self.run_streamed = to_streamed_response_wrapper(
            tasks.run_streamed,
        )
        self.send_message = to_streamed_response_wrapper(
            tasks.send_message,
        )
        self.stop = to_streamed_response_wrapper(
            tasks.stop,
        )

    @cached_property
    def screenshots(self) -> ScreenshotsResourceWithStreamingResponse:
        """Tasks API"""
        return ScreenshotsResourceWithStreamingResponse(self._tasks.screenshots)

    @cached_property
    def ui_states(self) -> UiStatesResourceWithStreamingResponse:
        """Tasks API"""
        return UiStatesResourceWithStreamingResponse(self._tasks.ui_states)


class AsyncTasksResourceWithStreamingResponse:
    def __init__(self, tasks: AsyncTasksResource) -> None:
        self._tasks = tasks

        self.retrieve = async_to_streamed_response_wrapper(
            tasks.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            tasks.list,
        )
        self.attach = async_to_streamed_response_wrapper(
            tasks.attach,
        )
        self.get_status = async_to_streamed_response_wrapper(
            tasks.get_status,
        )
        self.get_trajectory = async_to_streamed_response_wrapper(
            tasks.get_trajectory,
        )
        self.run = async_to_streamed_response_wrapper(
            tasks.run,
        )
        self.run_streamed = async_to_streamed_response_wrapper(
            tasks.run_streamed,
        )
        self.send_message = async_to_streamed_response_wrapper(
            tasks.send_message,
        )
        self.stop = async_to_streamed_response_wrapper(
            tasks.stop,
        )

    @cached_property
    def screenshots(self) -> AsyncScreenshotsResourceWithStreamingResponse:
        """Tasks API"""
        return AsyncScreenshotsResourceWithStreamingResponse(self._tasks.screenshots)

    @cached_property
    def ui_states(self) -> AsyncUiStatesResourceWithStreamingResponse:
        """Tasks API"""
        return AsyncUiStatesResourceWithStreamingResponse(self._tasks.ui_states)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .task import Task
from .._models import BaseModel
from .shared.pagination_meta import PaginationMeta

__all__ = ["TaskListResponse"]


class TaskListResponse(BaseModel):
    items: List[Task]
    """The paginated items"""

    pagination: PaginationMeta
    """Pagination metadata"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["PackageCredentials"]


class PackageCredentials(BaseModel):
    credential_names: List[str] = FieldInfo(alias="credentialNames")

    package_name: str = FieldInfo(alias="packageName")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["CredentialCreateParams", "Field"]


class CredentialCreateParams(TypedDict, total=False):
    credential_name: Required[Annotated[str, PropertyInfo(alias="credentialName")]]

    fields: Required[Iterable[Field]]


class Field(TypedDict, total=False):
    field_type: Required[
        Annotated[
            Literal["email", "username", "password", "api_token", "phone_number", "two_factor_secret", "backup_codes"],
            PropertyInfo(alias="fieldType"),
        ]
    ]

    value: Required[str]
